@@ -7,22 +7,35 @@
 import sys
 import types
 
-from twisted.python import log
-from twisted.internet import reactor, defer
+from twisted.internet import reactor
 from zope.interface import implements
 
 import txtorcon
 
+def finished(answer):
+    print "Answer:", answer
+    print "We could now do any sort of exciting thing we wanted..."
+    print "...but instead, we'll just exit."
+    reactor.stop()
+
+def query_changed_config(answer, state):
+    # now we'll ask for the ORPort back to prove it changed
+    state.protocol.get_conf("ORPort").addCallback(finished)
+
 def state_complete(state):
     print "We've completely booted up a TorState to a Tor version %s at PID %d" % (state.protocol.version, state.tor_pid)
 
     print "This Tor has the following %d Circuits:" % len(state.circuits)
     for c in state.circuits.values():
         print c
-    
-    print "We could now do any sort of exciting thing we wanted..."
-    print "...but instead, we'll just exit."
-    reactor.stop()
+
+    # we're using the global to demonstrate that the config we passed
+    # to launch_tor can be used after the Tor has been started
+    global config
+    config.ORPort = 9090
+    # "save" may be poorly-named API; it serializes the options to the
+    # running Tor (via SETCONF calls)
+    config.save().addCallback(query_changed_config, state)
 
 def setup_complete(proto):
     print "setup complete:",proto
@@ -35,11 +48,6 @@ def setup_failed(arg):
     print "SETUP FAILED",arg
     reactor.stop()
 
-def bootstrap(c):
-    conf = TorConfig(c)
-    conf.post_bootstrap.addCallback(setup_complete).addErrback(setup_failed)
-    print "Connection is live, bootstrapping state..."
-
 config = txtorcon.TorConfig()
 config.OrPort = 1234
 config.SocksPort = 9999
