@@ -5,6 +5,7 @@ from interface import IRouterContainer
 
 from txtorcon.util import find_keywords
 
+
 class Circuit(object):
     """
     Used by :class:`txtorcon.TorState` to represent one of Tor's circuits.
@@ -58,7 +59,8 @@ class Circuit(object):
 
     def __init__(self, routercontainer):
         """
-        :param routercontainer: should implement :class:`txtorcon.interface.IRouterContainer`
+        :param routercontainer: should implement
+        :class:`txtorcon.interface.IRouterContainer`
         """
         self.listeners = []
         self.router_container = IRouterContainer(routercontainer)
@@ -67,14 +69,15 @@ class Circuit(object):
         self.purpose = None
         self.id = None
         self.state = 'UNKNOWN'
-        
+        self.build_flags = []
+
     def listen(self, listener):
         if listener not in self.listeners:
             self.listeners.append(listener)
 
     def unlisten(self, listener):
         self.listeners.remove(listener)
-                    
+
     def update(self, args):
         ##print "Circuit.update:",args
         if self.id is None:
@@ -87,9 +90,11 @@ class Circuit(object):
         self.state = args[1]
 
         kw = find_keywords(args)
-        if kw.has_key('PURPOSE'):
+        if 'PURPOSE' in kw:
             self.purpose = kw['PURPOSE']
-            
+        if 'BUILD_FLAGS' in kw:
+            self.build_flags = kw['BUILD_FLAGS'].split(',')
+
         if self.state == 'LAUNCHED':
             self.path = []
             [x.circuit_launched(self) for x in self.listeners]
@@ -102,28 +107,49 @@ class Circuit(object):
 
         elif self.state == 'CLOSED':
             if len(self.streams) > 0:
-                log.err(RuntimeError("Circuit is %s but still has %d streams" % (self.state, len(self.streams))))
+                log.err(RuntimeError("Circuit is %s but still has %d streams" %
+                                     (self.state, len(self.streams))))
             [x.circuit_closed(self) for x in self.listeners]
 
         elif self.state == 'FAILED':
             if len(self.streams) > 0:
-                log.err(RuntimeError("Circuit is %s but still has %d streams" % (self.state, len(self.streams))))
+                log.err(RuntimeError("Circuit is %s but still has %d streams" %
+                                     (self.state, len(self.streams))))
             reason = 'unknown'
-            if kw.has_key('REASON'):
+            if 'REASON' in kw:
                 reason = kw['REASON']
             [x.circuit_failed(self, reason) for x in self.listeners]
 
     def update_path(self, path):
+        """
+        There are EXTENDED messages which don't include any routers at
+        all, and any of the EXTENDED messages may have some arbitrary
+        flags in them. So far, they're all upper-case and none start
+        with $ luckily. The routers in the path should all be
+        LongName-style router names (this depends on them starting
+        with $).
+
+        For further complication, it's possible to extend a circuit to
+        a router which isn't in the consensus. nickm via #tor thought
+        this might happen in the case of hidden services choosing a
+        rendevouz point not in the current consensus.
+        """
+        
         oldpath = self.path
         self.path = []
         for router in path:
             p = router[:41]
+            if p[0] != '$':
+                break
+
+            ## this will create a Router if we give it a router
+            ## LongName that doesn't yet exist
             router = self.router_container.router_from_id(p)
+            
             self.path.append(router)
             if len(self.path) > len(oldpath):
                 [x.circuit_extend(self, router) for x in self.listeners]
                 oldpath = self.path
-        
+
     def __str__(self):
-        #return "<Circuit %d %s [%s]>" % (self.id, self.state, ' '.join(map(lambda x: x.name, self.path)))
         return "<Circuit %d %s [%s] for %s>" % (self.id, self.state, ' '.join(map(lambda x: x.ip, self.path)), self.purpose)
