@@ -586,6 +586,14 @@ OK''')
         conf.hiddenservices[0].version = 3
         self.assertTrue(conf.needs_save())
         
+    def test_add_hidden_service_to_empty_config(self):
+        conf = TorConfig()
+        h = HiddenService(conf, '/fake/path', ['80 127.0.0.1:1234'], '', 3)
+        conf.hiddenservices.append(h)
+        self.assertEqual(len(conf.hiddenservices), 1)
+        self.assertEqual(h, conf.hiddenservices[0])
+        self.assertTrue(conf.needs_save())
+
     def test_multiple_startup_services(self):
         conf = TorConfig(FakeControlProtocol(['config/names=']))
         conf._setup_hidden_services('''HiddenServiceDir=/fake/path
@@ -696,7 +704,7 @@ class LaunchTorTests(unittest.TestCase):
         self.transport = proto_helpers.StringTransport()
         self.protocol.makeConnection(self.transport)
 
-    def setup_complete_no_errors(self, proto):
+    def setup_complete_no_errors(self, proto, config):
         todel = proto.to_delete
         self.assertTrue(len(todel) > 0)
         proto.processEnded(Failure(error.ProcessDone(0)))
@@ -704,6 +712,10 @@ class LaunchTorTests(unittest.TestCase):
         for f in todel:
             self.assertTrue(not os.path.exists(f))
 
+        ## make sure we set up the config to track the created tor
+        ## protocol connection
+        self.assertEquals(config.protocol, proto.tor_protocol)
+
     def setup_complete_fails(self, proto):
         todel = proto.to_delete
         self.assertTrue(len(todel) > 0)
@@ -745,7 +757,7 @@ class LaunchTorTests(unittest.TestCase):
         self.othertrans = trans
         creator = functools.partial(connector, self.protocol, self.transport)
         d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator)
-        d.addCallback(self.setup_complete_no_errors)
+        d.addCallback(self.setup_complete_no_errors, config)
         return d
         
     def check_setup_failure(self, fail):
@@ -1019,6 +1031,34 @@ OK''')
 
         return d
 
+    def test_explicit_data_dir(self):
+        ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123, '/mumble/mumble')
+        d = ep.listen(FakeProtocolFactory())
+
+        self.protocol.answers.append('''config/names=
+HiddenServiceOptions Virtual
+OK''')
+        self.protocol.answers.append('HiddenServiceOptions')
+        
+        self.config.bootstrap()
+
+        return d
+
+    def test_explicit_data_dir_valid_hostname(self):
+        datadir = tempfile.mkdtemp()
+        with open(os.path.join(datadir, 'hostname'), 'w') as f:
+            f.write('timaq4ygg2iegci7.onion')
+        with open(os.path.join(datadir, 'private_key'), 'w') as f:
+            f.write('foo\nbar')
+
+        try:
+            ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123, datadir)
+            self.assertEqual(ep.onion_uri, 'timaq4ygg2iegci7.onion')
+            self.assertEqual(ep.onion_private_key, 'foo\nbar')
+
+        finally:
+            shutil.rmtree(datadir, ignore_errors=True)
+
     def test_failure(self):
         self.reactor.failures = 2
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
