@@ -16,7 +16,8 @@ from txtorcon.addrmap import AddrMap
 from txtorcon.util import hmac_sha256, compare_via_hash
 from txtorcon.log import txtorlog
 
-from interface import ICircuitListener, ICircuitContainer, IStreamListener, IStreamAttacher, IRouterContainer, ITorControlProtocol
+from txtorcon.interface import ICircuitListener, ICircuitContainer, IStreamListener
+from txtorcon.interface import IStreamAttacher, IRouterContainer, ITorControlProtocol
 from spaghetti import FSM, State, Transition
 
 import os
@@ -33,6 +34,7 @@ import base64
 DEFAULT_VALUE = 'DEFAULT'
 DEBUG = False
 
+
 class TorProtocolError(RuntimeError):
     """
     Happens on 500-level responses in the protocol, almost certainly
@@ -65,7 +67,8 @@ class TorProtocolFactory(object):
 
     def __init__(self, password=None):
         """
-        Builds protocols to talk to a Tor client on the specified address. For example:
+        Builds protocols to talk to a Tor client on the specified
+        address. For example::
 
         TCP4ClientEndpoint(reactor, "localhost", 9051).connect(TorProtocolFactory())
         reactor.run()
@@ -89,6 +92,7 @@ class TorProtocolFactory(object):
         proto.factory = self
         return proto
 
+
 class Event(object):
     """
     A class representing one of the valid EVENTs that Tor
@@ -113,13 +117,29 @@ class Event(object):
         for cb in self.callbacks:
             cb(data)
 
-def parse_keywords(lines):
+def unquote(word):
+    if len(word) == 0:
+        return word
+    if word[0] == '"' and word[-1] == '"':
+        return word[1:-1]
+    elif word[0] == "'" and word[-1] == "'":
+        return word[1:-1]
+    return word
+
+def parse_keywords(lines, multiline_values=True):
     """
     Utility method to parse name=value pairs (GETINFO etc). Takes a
     string with newline-separated lines and expects at most one = sign
     per line. Accumulates multi-line values.
+
+    :param multiline_values:
+        The default is True which allows for multi-line values until a
+        line with the next = sign on it. So: '''Foo=bar\nBar'''
+        produces one key, 'Foo', with value 'bar\nBar' -- set to
+        False, there would be two keys: 'Foo' with value 'bar' and
+        'Bar' with value DEFAULT_VALUE.
     """
-    
+
     rtn = {}
     key = None
     value = ''
@@ -127,48 +147,58 @@ def parse_keywords(lines):
     for line in lines.split('\n'):
         if line.strip() == 'OK':
             continue
-        
+
         if '=' in line:
             if key:
-                if rtn.has_key(key):
+                if key in rtn:
                     if isinstance(rtn[key], types.ListType):
-                        rtn[key].append(value)
+                        rtn[key].append(unquote(value))
                     else:
-                        rtn[key] = [rtn[key], value]
+                        rtn[key] = [rtn[key], unquote(value)]
                 else:
-                    rtn[key] = value
-            (key, value) = line.split('=')
+                    rtn[key] = unquote(value)
+            (key, value) = line.split('=', 1)
 
         else:
             if key is None:
                 rtn[line.strip()] = DEFAULT_VALUE
-                #raise RuntimeError('Should have had a key by now: ' + lines)
+
+            elif multiline_values is False:
+                rtn[key] = value
+                rtn[line.strip()] = DEFAULT_VALUE
+                key = None
+                value = ''
+
             else:
                 value = value + '\n' + line
     if key:
-        if rtn.has_key(key):
+        if key in rtn:
             if isinstance(rtn[key], types.ListType):
-                rtn[key].append(value)
+                rtn[key].append(unquote(value))
             else:
-                rtn[key] = [rtn[key], value]
+                rtn[key] = [rtn[key], unquote(value)]
         else:
-            rtn[key] = value
+            rtn[key] = unquote(value)
     return rtn
 
+
 class TorControlProtocol(LineOnlyReceiver):
     """
-    This is the main class that talks to a Tor and implements the "raw" procotol.
+    This is the main class that talks to a Tor and implements the "raw"
+    procotol.
 
     This instance does not track state; see :class:`txtorcon.TorState`
     for the current state of all Circuits, Streams and Routers.
 
-    :meth:`txtorcon.TorState.build_circuit` allows you to build custom circuits.
+    :meth:`txtorcon.TorState.build_circuit` allows you to build custom
+    circuits.
 
-    :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used to listen for specific events.
+    :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used
+    to listen for specific events.
 
-    To see how circuit and stream listeners are used, see :class:`txtorcon.TorState`,
-    which is also the place to go if you wish to add your own stream
-    or circuit listeners.
+    To see how circuit and stream listeners are used, see
+    :class:`txtorcon.TorState`, which is also the place to go if you
+    wish to add your own stream or circuit listeners.
     """
 
     implements(ITorControlProtocol)
@@ -180,13 +210,15 @@ class TorControlProtocol(LineOnlyReceiver):
         """
 
         self.password = password
-        """If set, a password to use for authentication to Tor (default is to use COOKIE, however)."""
+        """If set, a password to use for authentication to Tor
+        (default is to use COOKIE, however)."""
 
         self.version = None
         """Version of Tor we've connected to."""
 
         self.is_owned = None
-        """If not None, this is the PID of the Tor process we own (TAKEOWNERSHIP, etc)."""
+        """If not None, this is the PID of the Tor process we own
+        (TAKEOWNERSHIP, etc)."""
 
         self.events = {}
         """events we've subscribed to (keyed by name like "GUARD", "STREAM")"""
@@ -276,7 +308,7 @@ class TorControlProtocol(LineOnlyReceiver):
         the GETINFO command. See :meth:`getinfo <txtorcon.TorControlProtocol.get_info>`
         """
         info = ' '.join(map(lambda x: str(x), list(args)))
-        return self.queue_command('GETINFO %s'%info)
+        return self.queue_command('GETINFO %s' % info)
 
     def get_info_incremental(self, key, line_cb):
         """
@@ -286,7 +318,7 @@ class TorControlProtocol(LineOnlyReceiver):
         See :meth:`getinfo <txtorcon.TorControlProtocol.get_info>`
         """
 
-        return self.queue_command('GETINFO %s'%key, line_cb)
+        return self.queue_command('GETINFO %s' % key, line_cb)
 
     ## The following methods are the main TorController API and
     ## probably the most interesting for users.
@@ -351,7 +383,8 @@ class TorControlProtocol(LineOnlyReceiver):
         to) set the key 'foo' to value 'bar'.
 
         :return: a ``Deferred`` that will callback with the response
-            ('OK') or errback with the error code and message (e.g. ``"552 Unrecognized option: Unknown option 'foo'.  Failing."``)
+            ('OK') or errback with the error code and message (e.g.
+            ``"552 Unrecognized option: Unknown option 'foo'.  Failing."``)
         """
         if len(args) % 2:
             d = defer.Deferred()
@@ -360,12 +393,13 @@ class TorControlProtocol(LineOnlyReceiver):
         strargs = map(lambda x: str(x), args)
         keys = [strargs[i] for i in range(0, len(strargs), 2)]
         values = [strargs[i] for i in range(1, len(strargs), 2)]
+        
         def maybe_quote(s):
             if ' ' in s:
                 return '"%s"' % s
             return s
         values = map(maybe_quote, values)
-        args = ' '.join(map(lambda x,y:'%s=%s'%(x,y), keys, values))
+        args = ' '.join(map(lambda x, y: '%s=%s' % (x, y), keys, values))
         return self.queue_command('SETCONF ' + args)
 
     def signal(self, nm):
@@ -383,7 +417,9 @@ class TorControlProtocol(LineOnlyReceiver):
 
     def add_event_listener(self, evt, callback):
         """
-        :param evt: event name, see also :var:`txtorcon.TorControlProtocol.events` .keys()
+        :param evt: event name, see also
+        :var:`txtorcon.TorControlProtocol.events` .keys()
+        
         Add a listener to an Event object. This may be called multiple
         times for the same event. If it's the first listener, a new
         SETEVENTS call will be initiated to Tor.
@@ -403,7 +439,7 @@ class TorControlProtocol(LineOnlyReceiver):
             except:
                 raise RuntimeError("Unknown event type: " + evt)
 
-        if not self.events.has_key(evt.name):
+        if evt.name not in self.events:
             self.events[evt.name] = evt
             self.queue_command('SETEVENTS %s' % ' '.join(self.events.keys()))
         evt.listen(callback)
@@ -437,7 +473,8 @@ class TorControlProtocol(LineOnlyReceiver):
 
     def queue_command(self, cmd, arg=None):
         """
-        returns a Deferred which will fire with the response data when we get it
+        returns a Deferred which will fire with the response data when
+        we get it
         """
 
         d = defer.Deferred()
@@ -453,11 +490,11 @@ class TorControlProtocol(LineOnlyReceiver):
         """
         :api:`twisted.protocols.basic.LineOnlyReceiver` API
         """
-        
+
         if DEBUG:
-            self.debuglog.write(line+'\n')
+            self.debuglog.write(line + '\n')
             self.debuglog.flush()
-            
+
         self.fsm.process(line)
 
     def connectionMade(self):
@@ -469,11 +506,11 @@ class TorControlProtocol(LineOnlyReceiver):
         """
         Internal method to deal with 600-level responses.
         """
-        
+
         firstline = rest[:rest.find('\n')]
         args = firstline.split()
-        if self.events.has_key(args[0]):
-            self.events[args[0]].got_update(rest[len(args[0])+1:])
+        if args[0] in self.events:
+            self.events[args[0]].got_update(rest[len(args[0]) + 1:])
             return
 
         raise RuntimeError("Wasn't listening for event of type " + args[0])
@@ -494,7 +531,7 @@ class TorControlProtocol(LineOnlyReceiver):
 
             if DEBUG:
                 #print "NOTIFY",code,rest
-                self.debuglog.write(cmd+'\n')
+                self.debuglog.write(cmd + '\n')
                 self.debuglog.flush()
 
             self.transport.write(cmd + '\r\n')
@@ -521,7 +558,7 @@ class TorControlProtocol(LineOnlyReceiver):
         ## FIXME put string in global. or something.
         expected_server_hash = hmac_sha256("Tor safe cookie authentication server-to-controller hash",
                                            self.cookie_data + self.client_nonce + server_nonce)
-        
+
         if not compare_via_hash(expected_server_hash, server_hash):
             raise RuntimeError('Server hash not expected; wanted "%s" and got "%s".' % (base64.b16encode(expected_server_hash),
                                                                                         base64.b16encode(server_hash)))
@@ -533,7 +570,8 @@ class TorControlProtocol(LineOnlyReceiver):
 
     def _do_authenticate(self, protoinfo):
         """
-        Callback on PROTOCOLINFO to actually authenticate once we know what's supported.
+        Callback on PROTOCOLINFO to actually authenticate once we know
+        what's supported.
         """
 
         methods = None
@@ -672,7 +710,7 @@ class TorControlProtocol(LineOnlyReceiver):
 
     def _accumulate_response(self, line):
         "for FSM"
-        if self.command[2] != None:
+        if self.command and self.command[2] != None:
             self.command[2](line[4:])
 
         else:
@@ -694,7 +732,7 @@ class TorControlProtocol(LineOnlyReceiver):
         "for FSM"
 #        print "BCAST",line
         if len(line) > 3:
-            if self.code >= 200 and self.code < 300 and self.command[2] != None:
+            if self.code >= 200 and self.code < 300 and self.command and self.command[2] != None:
                 self.command[2](line[4:])
                 resp = ''
 
@@ -704,6 +742,8 @@ class TorControlProtocol(LineOnlyReceiver):
             resp = self.response
         self.response = ''
         if self.code >= 200 and self.code < 300:
+            if self.defer is None:
+                raise RuntimeError("Got a response, but didn't issue a command.");
             self.defer.callback(resp)
         elif self.code >= 500 and self.code < 600:
             err = TorProtocolError(self.code, resp)
